/********************************************************************************

	GnucNS - Network Simulator
    Copyright (C) 2003 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: swabby@c0re.net

********************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using GnucNS.Simulation;

namespace GnucNS.Interface
{
	/// <summary>
	/// 
	/// </summary>
	public class FormMain : System.Windows.Forms.Form
	{
		bool SimRunning = false;

		public SimSetup   TestSetup;
		public SimNetwork TestNet;

		public FormNetwork  NetworkView;
		public FormWebCache WebCacheView;
		public FormSetup	SetupView;
		public FormStatsGeneral	StatsGeneralView;
		public FormStatsHorizon	StatsHorizonView;
		public FormStatsTraffic	StatsTrafficView;

		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem MenuViewWebCache;
		private System.Windows.Forms.MenuItem MenuFile;
		private System.Windows.Forms.MenuItem MenuView;
		private System.Windows.Forms.MenuItem MenuFileExit;
		private System.Windows.Forms.MenuItem MenuViewNetwork;
		private System.Windows.Forms.MenuItem MenuHelp;
		private System.Windows.Forms.MenuItem MenuHelpAbout;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button ButtonStop;
		private System.Windows.Forms.Button ButtonStep;
		private System.Windows.Forms.Button ButtonRun;
		private System.Windows.Forms.Label LabelElapsed;
		private System.Windows.Forms.MenuItem MenuFileSetup;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.Button ButtonReset;
		private System.Windows.Forms.MenuItem MenuStats;
		private System.Windows.Forms.MenuItem MenuStatsGeneral;
		private System.Windows.Forms.MenuItem MenuStatsHorizon;
		private System.Windows.Forms.MenuItem MenuStatsTraffic;


		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public FormMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();


			Application.Idle += new System.EventHandler(this.Application_Idle);
			

			TestSetup = new SimSetup();
			TestNet = new SimNetwork(TestSetup);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.MenuFile = new System.Windows.Forms.MenuItem();
			this.MenuFileSetup = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.MenuFileExit = new System.Windows.Forms.MenuItem();
			this.MenuView = new System.Windows.Forms.MenuItem();
			this.MenuViewNetwork = new System.Windows.Forms.MenuItem();
			this.MenuViewWebCache = new System.Windows.Forms.MenuItem();
			this.MenuStats = new System.Windows.Forms.MenuItem();
			this.MenuStatsGeneral = new System.Windows.Forms.MenuItem();
			this.MenuStatsHorizon = new System.Windows.Forms.MenuItem();
			this.MenuStatsTraffic = new System.Windows.Forms.MenuItem();
			this.MenuHelp = new System.Windows.Forms.MenuItem();
			this.MenuHelpAbout = new System.Windows.Forms.MenuItem();
			this.label1 = new System.Windows.Forms.Label();
			this.LabelElapsed = new System.Windows.Forms.Label();
			this.ButtonStop = new System.Windows.Forms.Button();
			this.ButtonStep = new System.Windows.Forms.Button();
			this.ButtonRun = new System.Windows.Forms.Button();
			this.ButtonReset = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.MenuFile,
																					  this.MenuView,
																					  this.MenuStats,
																					  this.MenuHelp});
			// 
			// MenuFile
			// 
			this.MenuFile.Index = 0;
			this.MenuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.MenuFileSetup,
																					 this.menuItem3,
																					 this.MenuFileExit});
			this.MenuFile.Text = "File";
			// 
			// MenuFileSetup
			// 
			this.MenuFileSetup.Index = 0;
			this.MenuFileSetup.Text = "Setup";
			this.MenuFileSetup.Click += new System.EventHandler(this.MenuFileSetup_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 1;
			this.menuItem3.Text = "-";
			// 
			// MenuFileExit
			// 
			this.MenuFileExit.Index = 2;
			this.MenuFileExit.Text = "Exit";
			this.MenuFileExit.Click += new System.EventHandler(this.MenuFileExit_Click);
			// 
			// MenuView
			// 
			this.MenuView.Index = 1;
			this.MenuView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.MenuViewNetwork,
																					 this.MenuViewWebCache});
			this.MenuView.Text = "View";
			// 
			// MenuViewNetwork
			// 
			this.MenuViewNetwork.Index = 0;
			this.MenuViewNetwork.Text = "Network";
			this.MenuViewNetwork.Click += new System.EventHandler(this.MenuViewNetwork_Click);
			// 
			// MenuViewWebCache
			// 
			this.MenuViewWebCache.Index = 1;
			this.MenuViewWebCache.Text = "WebCache";
			this.MenuViewWebCache.Click += new System.EventHandler(this.MenuViewWebCache_Click);
			// 
			// MenuStats
			// 
			this.MenuStats.Index = 2;
			this.MenuStats.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.MenuStatsGeneral,
																					  this.MenuStatsHorizon,
																					  this.MenuStatsTraffic});
			this.MenuStats.Text = "Stats";
			// 
			// MenuStatsGeneral
			// 
			this.MenuStatsGeneral.Index = 0;
			this.MenuStatsGeneral.Text = "General";
			this.MenuStatsGeneral.Click += new System.EventHandler(this.MenuStatsGeneral_Click);
			// 
			// MenuStatsHorizon
			// 
			this.MenuStatsHorizon.Index = 1;
			this.MenuStatsHorizon.Text = "Horizon";
			this.MenuStatsHorizon.Click += new System.EventHandler(this.MenuStatsHorizon_Click);
			// 
			// MenuStatsTraffic
			// 
			this.MenuStatsTraffic.Index = 2;
			this.MenuStatsTraffic.Text = "Traffic";
			this.MenuStatsTraffic.Click += new System.EventHandler(this.MenuStatsTraffic_Click);
			// 
			// MenuHelp
			// 
			this.MenuHelp.Index = 3;
			this.MenuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.MenuHelpAbout});
			this.MenuHelp.Text = "Help";
			// 
			// MenuHelpAbout
			// 
			this.MenuHelpAbout.Index = 0;
			this.MenuHelpAbout.Text = "About";
			this.MenuHelpAbout.Click += new System.EventHandler(this.MenuHelpAbout_Click);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(24, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(224, 16);
			this.label1.TabIndex = 3;
			this.label1.Text = "Simulation Control";
			// 
			// LabelElapsed
			// 
			this.LabelElapsed.Location = new System.Drawing.Point(32, 80);
			this.LabelElapsed.Name = "LabelElapsed";
			this.LabelElapsed.Size = new System.Drawing.Size(240, 16);
			this.LabelElapsed.TabIndex = 4;
			this.LabelElapsed.Text = "Elapsed Time: 0 Seconds";
			// 
			// ButtonStop
			// 
			this.ButtonStop.Location = new System.Drawing.Point(32, 40);
			this.ButtonStop.Name = "ButtonStop";
			this.ButtonStop.Size = new System.Drawing.Size(64, 23);
			this.ButtonStop.TabIndex = 5;
			this.ButtonStop.Text = "Stop";
			this.ButtonStop.Click += new System.EventHandler(this.ButtonStop_Click);
			// 
			// ButtonStep
			// 
			this.ButtonStep.Location = new System.Drawing.Point(104, 40);
			this.ButtonStep.Name = "ButtonStep";
			this.ButtonStep.Size = new System.Drawing.Size(64, 23);
			this.ButtonStep.TabIndex = 6;
			this.ButtonStep.Text = "Step";
			this.ButtonStep.Click += new System.EventHandler(this.ButtonStep_Click);
			// 
			// ButtonRun
			// 
			this.ButtonRun.Location = new System.Drawing.Point(176, 40);
			this.ButtonRun.Name = "ButtonRun";
			this.ButtonRun.Size = new System.Drawing.Size(64, 23);
			this.ButtonRun.TabIndex = 7;
			this.ButtonRun.Text = "Run";
			this.ButtonRun.Click += new System.EventHandler(this.ButtonRun_Click);
			// 
			// ButtonReset
			// 
			this.ButtonReset.Location = new System.Drawing.Point(80, 112);
			this.ButtonReset.Name = "ButtonReset";
			this.ButtonReset.Size = new System.Drawing.Size(112, 23);
			this.ButtonReset.TabIndex = 8;
			this.ButtonReset.Text = "Reset Simulation";
			this.ButtonReset.Click += new System.EventHandler(this.ButtonReset_Click);
			// 
			// FormMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(258, 147);
			this.Controls.Add(this.ButtonReset);
			this.Controls.Add(this.ButtonRun);
			this.Controls.Add(this.ButtonStep);
			this.Controls.Add(this.ButtonStop);
			this.Controls.Add(this.LabelElapsed);
			this.Controls.Add(this.label1);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Menu = this.mainMenu1;
			this.Name = "FormMain";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "GnucNS";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new FormMain());
		}


		private void Application_Idle(object sender, System.EventArgs e)
		{
			if( SimRunning )
				StepSimulation();
		}

		private void StepSimulation()
		{
			TestNet.RunSecond();
				
			if(NetworkView != null)
				NetworkView.PanelDisplay.InvalidateData();

			LabelElapsed.Text = string.Format("Elapsed Time: {0} Hours, {1} Mins, {2} Secs", TestNet.NetworkAlive / 3600, (TestNet.NetworkAlive % 3600) / 60, TestNet.NetworkAlive % 60);
		}

		private void MenuFileExit_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		private void MenuFileSetup_Click(object sender, System.EventArgs e)
		{
			if(SetupView == null)
				SetupView = new FormSetup(this);
			
			SetupView.Show();
		}

		private void MenuViewNetwork_Click(object sender, System.EventArgs e)
		{
			if(NetworkView == null)
				NetworkView = new FormNetwork(this);
			
			NetworkView.Show();
		}

		private void MenuViewWebCache_Click(object sender, System.EventArgs e)
		{
			if(WebCacheView == null)
				WebCacheView = new FormWebCache(this);
			
			WebCacheView.Show();
		}

		private void MenuHelpAbout_Click(object sender, System.EventArgs e)
		{
			FormAbout aboutView = new FormAbout();
			aboutView.ShowDialog(this);
		}

		private void ButtonStop_Click(object sender, System.EventArgs e)
		{
			SimRunning = false;
		}

		private void ButtonStep_Click(object sender, System.EventArgs e)
		{
			StepSimulation();
		}

		private void ButtonRun_Click(object sender, System.EventArgs e)
		{
			SimRunning = true;
		}

		private void ButtonReset_Click(object sender, System.EventArgs e)
		{
			if( DialogResult.Yes == MessageBox.Show(this, "Reset Simulation?", "Reset", MessageBoxButtons.YesNo) )
			{
				SimRunning = false;
		
				TestNet = new SimNetwork(TestSetup);

				if(NetworkView != null)
					NetworkView.PanelDisplay.InvalidateData();

				LabelElapsed.Text = "Elapsed Time: 0 Hours, 0 Mins, 0 Secs";
		
			}
		}

		private void MenuStatsGeneral_Click(object sender, System.EventArgs e)
		{
			if(StatsGeneralView == null)
				StatsGeneralView = new FormStatsGeneral(this);
			
			StatsGeneralView.Show();
		}

		private void MenuStatsHorizon_Click(object sender, System.EventArgs e)
		{
			if(StatsHorizonView == null)
				StatsHorizonView = new FormStatsHorizon(this);
			
			StatsHorizonView.Show();
		}

		private void MenuStatsTraffic_Click(object sender, System.EventArgs e)
		{
			if(StatsTrafficView == null)
				StatsTrafficView = new FormStatsTraffic(this);
			
			StatsTrafficView.Show();
		}

		

		

	}
}
