/********************************************************************************

	GnucNS - Network Simulator
    Copyright (C) 2003 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: swabby@c0re.net

********************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using GnucNS.Simulation;

using System.Diagnostics;

namespace GnucNS.Interface
{
	/// <summary>
	/// Summary description for FormNetwork.
	/// </summary>
	public class FormNetwork : System.Windows.Forms.Form
	{
		public FormMain MainView;

		public NodeInfo DialogInfo;
		
		
		SimNetwork TestNet;

		enum SelectionColor { Same, Hops };
		SelectionColor DisplaySelection = SelectionColor.Same;

		enum Views { Ellipse, Grid };
		Views DisplayView = Views.Ellipse; 

		Color[] HopColors = {Color.Red, Color.Orange, Color.Yellow, Color.Green, Color.Blue, Color.Purple, Color.Brown};

		Bitmap DisplayBuffer;

		int	 NodeDiameter = 6;
		Hashtable PointTable = new Hashtable();

		int ClickedNodeID = 0;


		public GnucNS.Interface.PanelEx PanelDisplay;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem MenuView;
		private System.Windows.Forms.MenuItem MenuViewEllipse;
		private System.Windows.Forms.MenuItem MenuViewGrid;
		private System.Windows.Forms.MenuItem MenuSelection;
		private System.Windows.Forms.MenuItem MenuSelectionSame;
		private System.Windows.Forms.MenuItem MenuSelectionHops;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public FormNetwork(FormMain parent)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			MainView = parent;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			MainView.NetworkView = null;

			if(DialogInfo != null)
				DialogInfo.Close();

			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.PanelDisplay = new GnucNS.Interface.PanelEx();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.MenuView = new System.Windows.Forms.MenuItem();
			this.MenuViewEllipse = new System.Windows.Forms.MenuItem();
			this.MenuViewGrid = new System.Windows.Forms.MenuItem();
			this.MenuSelection = new System.Windows.Forms.MenuItem();
			this.MenuSelectionSame = new System.Windows.Forms.MenuItem();
			this.MenuSelectionHops = new System.Windows.Forms.MenuItem();
			this.SuspendLayout();
			// 
			// PanelDisplay
			// 
			this.PanelDisplay.BackColor = System.Drawing.Color.White;
			this.PanelDisplay.Dock = System.Windows.Forms.DockStyle.Fill;
			this.PanelDisplay.Location = new System.Drawing.Point(0, 0);
			this.PanelDisplay.Name = "PanelDisplay";
			this.PanelDisplay.Size = new System.Drawing.Size(292, 273);
			this.PanelDisplay.TabIndex = 1;
			this.PanelDisplay.Resize += new System.EventHandler(this.PanelDisplay_Resize);
			this.PanelDisplay.Paint += new System.Windows.Forms.PaintEventHandler(this.PanelDisplay_Paint);
			this.PanelDisplay.MouseDown += new System.Windows.Forms.MouseEventHandler(this.PanelDisplay_MouseDown);
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.MenuView,
																					  this.MenuSelection});
			// 
			// MenuView
			// 
			this.MenuView.Index = 0;
			this.MenuView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.MenuViewEllipse,
																					 this.MenuViewGrid});
			this.MenuView.Text = "View";
			// 
			// MenuViewEllipse
			// 
			this.MenuViewEllipse.Index = 0;
			this.MenuViewEllipse.Text = "Ellipse";
			this.MenuViewEllipse.Click += new System.EventHandler(this.MenuViewEllipse_Click);
			// 
			// MenuViewGrid
			// 
			this.MenuViewGrid.Index = 1;
			this.MenuViewGrid.Text = "Grid";
			this.MenuViewGrid.Click += new System.EventHandler(this.MenuViewGrid_Click);
			// 
			// MenuSelection
			// 
			this.MenuSelection.Index = 1;
			this.MenuSelection.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.MenuSelectionSame,
																						  this.MenuSelectionHops});
			this.MenuSelection.Text = "Selection";
			// 
			// MenuSelectionSame
			// 
			this.MenuSelectionSame.Index = 0;
			this.MenuSelectionSame.Text = "Color Same";
			this.MenuSelectionSame.Click += new System.EventHandler(this.MenuSelectionSame_Click);
			// 
			// MenuSelectionHops
			// 
			this.MenuSelectionHops.Index = 1;
			this.MenuSelectionHops.Text = "Color Hops";
			this.MenuSelectionHops.Click += new System.EventHandler(this.MenuSelectionHops_Click);
			// 
			// FormNetwork
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Controls.Add(this.PanelDisplay);
			this.Menu = this.mainMenu1;
			this.Name = "FormNetwork";
			this.Text = "Network";
			this.ResumeLayout(false);

		}
		#endregion

		

		private void PanelDisplay_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{			
			
			TestNet = MainView.TestNet;
			SimSetup testSetup = MainView.TestSetup;

			if(PanelDisplay.Width == 0 || PanelDisplay.Height == 0)
				return;

			if(DisplayBuffer == null )
				DisplayBuffer = new Bitmap(PanelDisplay.Width, PanelDisplay.Height);

			if( !PanelDisplay.Redraw )
			{
				Graphics sameDisplay = e.Graphics;
				sameDisplay.DrawImage(DisplayBuffer, 0, 0);
				return;
			}
			PanelDisplay.Redraw = false;
		
			Graphics buffer = Graphics.FromImage(DisplayBuffer);

			buffer.Clear(Color.White);
			buffer.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;


			IDictionaryEnumerator nodeEnum = TestNet.NodeTable.GetEnumerator();
		
			int ultrapeerCount = 0;
			while( nodeEnum.MoveNext() )
				if( !((SimNode) nodeEnum.Value).ChildMode )
					ultrapeerCount++;

			nodeEnum.Reset();

			// Ellipse
			if( DisplayView == Views.Ellipse )
			{
				int nodeIndex = 0;

				while( nodeEnum.MoveNext() )
				{
					if( ((SimNode) nodeEnum.Value).ChildMode )
						continue;

					double piPos = Math.PI*2 / ultrapeerCount * nodeIndex;

					Point nodePoint = new Point(Convert.ToInt32( (Math.Cos(piPos) + 1) * ( (PanelDisplay.Width - 40) / 2)) + 10,
						Convert.ToInt32( (Math.Sin(piPos) + 1) * ( (PanelDisplay.Height - 20) / 2)) + 10);

					PointTable[((SimNode) nodeEnum.Value).NodeID] = nodePoint;

					nodeIndex++;
				}	
			}

			// Grid
			if( DisplayView == Views.Grid )
			{
				int segs = 1;
				while(Math.Pow(segs - 1, 2) < ultrapeerCount)
					segs++;

				for(int x = 1; x < segs; x++)
					for(int y = 1; y < segs; y++)
					{
						while( nodeEnum.MoveNext() )
						{
							if( ((SimNode) nodeEnum.Value).ChildMode )
								continue;

							Point nodePoint = new Point( (PanelDisplay.Width  / segs) * y,
								(PanelDisplay.Height / segs) * x);

							PointTable[((SimNode) nodeEnum.Value).NodeID] = nodePoint;

							break;
						}
					}
			}


			// Draw Connections
			Pen nodeTryPen     = new Pen(new SolidBrush(Color.DarkGoldenrod), 1);
			Pen nodeConnectPen = new Pen(new SolidBrush(Color.DarkGreen),     1);

			nodeEnum.Reset();
			while( nodeEnum.MoveNext() )
			{
				SimNode currentNode = (SimNode) nodeEnum.Value;

				if( currentNode.ChildMode )
					continue;

				int tryNodeID = currentNode.AttemptID;

				if( tryNodeID > 0 && TestNet.NodeTable[tryNodeID] != null && PointTable[tryNodeID] != null)
					buffer.DrawLine(nodeTryPen, (Point) PointTable[currentNode.NodeID], (Point) PointTable[tryNodeID]);


				foreach( int nodeID in currentNode.Connections )
					// Only draw lines once
					if( ((SimNode) nodeEnum.Value).NodeID < nodeID && TestNet.NodeTable[nodeID] != null)
						buffer.DrawLine(nodeConnectPen, (Point) PointTable[currentNode.NodeID], (Point) PointTable[nodeID]);
			}

		
			// Draw Selected Nodes and Connections
			SimNode selectedNode = (SimNode) TestNet.NodeTable[ ClickedNodeID ];
			int collisions = 0;
			Hashtable coveredNodes = new Hashtable(); // key nodeID, value hops

			if( selectedNode != null)
			{
				ArrayList pendingNodes = new ArrayList();
				ArrayList linesArry = new ArrayList();

				int hops = 0;
				pendingNodes.Add( new NodeLink(selectedNode.NodeID, 0));

				while(hops <= testSetup.NetworkTTL && pendingNodes.Count > 0)
				{
					ArrayList nextNodes = new ArrayList();

					foreach( NodeLink link in pendingNodes)
					{
						SimNode node = (SimNode) TestNet.NodeTable[link.NodeID];

						if( coveredNodes[link.NodeID] == null && node != null)
						{
							coveredNodes[link.NodeID] = hops;

							if(hops < testSetup.NetworkTTL)
								foreach( int nextNodeID in node.Connections)
								{
									SimNode nextNode = (SimNode) TestNet.NodeTable[nextNodeID];
								
									if( nextNode != null && nextNodeID != link.LinkID)
									{	
										linesArry.Add( new LineEx( (Point) PointTable[link.NodeID], (Point) PointTable[nextNodeID], GetHopColor(hops + 1)));
									
										nextNodes.Add( new NodeLink(nextNodeID, link.NodeID) );
									}
								}
						}
						else if(link.NodeID != 0)
						{
							collisions++;
						}	
					}

					pendingNodes = nextNodes;
					hops++;
				}

				// Reverse and draw lineArray
				linesArry.Reverse();

				foreach( LineEx line in linesArry)
				{
					if( DisplaySelection == SelectionColor.Same )
						buffer.DrawLine( new Pen(Color.Blue, 1), line.P1, line.P2);
					else
						buffer.DrawLine( new Pen(line.LineColor, 1), line.P1, line.P2);
				}
			}


			// Draw Nodes
			Point nodePos;
			Size nodeSize			= new Size(NodeDiameter, NodeDiameter);
	
			Pen firePen				= new Pen(Color.Red);
			int firewallCount = 0;

			nodeEnum.Reset();
			while( nodeEnum.MoveNext() )
			{
				SimNode currentNode = (SimNode) nodeEnum.Value;
				
				if( currentNode.Firewalled )
					firewallCount++;

				if( currentNode.ChildMode )
					continue;

				nodePos = (Point) PointTable[currentNode.NodeID];
				nodePos.X -= (NodeDiameter / 2);
				nodePos.Y -= (NodeDiameter / 2);

				SolidBrush nodeBrush;
				if( coveredNodes[currentNode.NodeID] != null)
				{
					if( DisplaySelection == SelectionColor.Same )
					{
						if(currentNode == selectedNode)
							nodeBrush = new SolidBrush( Color.Orange );
						else
							nodeBrush = new SolidBrush( Color.Blue );
					}	
					else
						nodeBrush = new SolidBrush( GetHopColor((int) coveredNodes[currentNode.NodeID]));
				}
				else
					nodeBrush = new SolidBrush(Color.Black);

				
				buffer.FillEllipse(nodeBrush, new Rectangle( nodePos, nodeSize));


				if( currentNode.Firewalled)
					buffer.DrawEllipse(firePen, nodePos.X - 2, nodePos.Y - 2, nodeSize.Width + 4, nodeSize.Height + 4);

				if( currentNode.Children.Count > 0 )
				{
					Font nodeFont = new Font("Tahoma", 8);
					nodePos.X += 8;

					SolidBrush bgBrush = new SolidBrush( Color.FromArgb(128, 255, 255, 255) );
					buffer.FillRectangle(bgBrush, nodePos.X, nodePos.Y, 16, 16);
					buffer.DrawString( string.Format("{0}", currentNode.Children.Count), nodeFont, nodeBrush, nodePos);
				}
			}

	
			this.Text = string.Format("Network - {0} Nodes, {1} Firewalled", TestNet.NodeTable.Count, firewallCount);


			if( selectedNode != null)
				if(DialogInfo != null && ultrapeerCount > 0)
					DialogInfo.SetData(selectedNode.NodeID, coveredNodes.Count * 100 / ultrapeerCount, collisions);


			// Copy buffer to display
			Graphics display = e.Graphics;
			display.DrawImage(DisplayBuffer, 0, 0);
		}

		private void PanelDisplay_Resize(object sender, System.EventArgs e)
		{
			if(PanelDisplay.Width > 0 && PanelDisplay.Height > 0)
			{
				DisplayBuffer = new Bitmap(PanelDisplay.Width, PanelDisplay.Height);
				PanelDisplay.InvalidateData();
			}
		}

		private void MenuViewEllipse_Click(object sender, System.EventArgs e)
		{
			DisplayView = Views.Ellipse;

			PanelDisplay.InvalidateData();
		}

		private void MenuViewGrid_Click(object sender, System.EventArgs e)
		{
			DisplayView = Views.Grid;

			PanelDisplay.InvalidateData();
		}

		private void PanelDisplay_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(e.Button != MouseButtons.Left)
				return;

			int PreviousID = ClickedNodeID;

			ClickedNodeID = 0;

			IDictionaryEnumerator enumPoint = PointTable.GetEnumerator();
			while( enumPoint.MoveNext() )
			{
				Rectangle nodeRect = new Rectangle( ((Point) enumPoint.Value).X - (NodeDiameter / 2) - 2, ((Point) enumPoint.Value).Y - (NodeDiameter / 2) - 2, NodeDiameter + 4, NodeDiameter + 4);

				if( nodeRect.Contains( e.X, e.Y) )
				{
					ClickedNodeID = (int) enumPoint.Key;
					break;
				}

				if(DialogInfo == null)
					DialogInfo = new NodeInfo(this);

				DialogInfo.SetData(ClickedNodeID, 0, 0);
				DialogInfo.Show();
			}

			if( PreviousID != ClickedNodeID)
				PanelDisplay.InvalidateData();
			
		}

		private void MenuSelectionSame_Click(object sender, System.EventArgs e)
		{
			DisplaySelection = SelectionColor.Same;
			PanelDisplay.InvalidateData();
		}	

		private void MenuSelectionHops_Click(object sender, System.EventArgs e)
		{
			DisplaySelection = SelectionColor.Hops;
			PanelDisplay.InvalidateData();
		}
		
		private Color GetHopColor(int hop)
		{
			if(hop == 0)
				return Color.Black;

			if(hop > HopColors.Length)
				return Color.Brown;

			return HopColors[hop - 1];
			
		}
	}
	

	struct LineEx
	{
		public Point P1;
		public Point P2;
		public Color LineColor;

		public LineEx(Point p1, Point p2, Color lineColor)
		{
			P1 = p1;
			P2 = p2;
			LineColor = lineColor;
			
		}
	}
}
