/********************************************************************************

	GnucNS - Network Simulator
    Copyright (C) 2003 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: swabby@c0re.net

********************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using System.IO;
using System.Text;
using System.Xml;
using GnucNS.Simulation;

namespace GnucNS.Interface
{
	/// <summary>
	/// Summary description for FormSetup.
	/// </summary>
	public class FormSetup : System.Windows.Forms.Form
	{
		FormMain MainView;


		SimSetup NewSetup;

		enum IndexVars  {
						StartingNodes,
						StartingWebCaches,
						Break1,
						NetworkTTL,
						NetworkInflux,
						NetworkOutflux,
						NetworkCapacity,
						NetworkFirewalled,
						Break2,
						NodeConnectMin,
						NodeConnectMax,
						NodeAvoidTriangles,
						Break3,
						TrafficSim,
						NodeBandwidthMin,
						NodeBandwidthMax,
						NodeQueryFreq,
						Break4,
						UltrapeerSim,
						NodeLeafMin,
						NodeLeafMax,
						LeafModeConnects
					};

		private System.Windows.Forms.ListView ListSetup;
		private System.Windows.Forms.Button ButtonApply;
		private System.Windows.Forms.Button ButtonOK;
		private System.Windows.Forms.Button ButtonSave;
		private System.Windows.Forms.Button ButtonLoad;
		private System.Windows.Forms.Button ButtonCancel;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public FormSetup(FormMain parent)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			MainView = parent;

			NewSetup = (SimSetup) MainView.TestSetup.Clone();
			
			ListSetup.Columns.Add("Variable", -2, HorizontalAlignment.Left);
			ListSetup.Columns.Add("Value", -2, HorizontalAlignment.Right);
			ListSetup.Columns.Add("Description", -2, HorizontalAlignment.Left);

			RefreshList();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			MainView.SetupView = null;

			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.ListSetup = new System.Windows.Forms.ListView();
			this.ButtonApply = new System.Windows.Forms.Button();
			this.ButtonOK = new System.Windows.Forms.Button();
			this.ButtonSave = new System.Windows.Forms.Button();
			this.ButtonLoad = new System.Windows.Forms.Button();
			this.ButtonCancel = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// ListSetup
			// 
			this.ListSetup.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.ListSetup.FullRowSelect = true;
			this.ListSetup.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.ListSetup.Location = new System.Drawing.Point(16, 40);
			this.ListSetup.MultiSelect = false;
			this.ListSetup.Name = "ListSetup";
			this.ListSetup.Size = new System.Drawing.Size(448, 188);
			this.ListSetup.TabIndex = 0;
			this.ListSetup.View = System.Windows.Forms.View.Details;
			this.ListSetup.DoubleClick += new System.EventHandler(this.ListSetup_DoubleClick);
			// 
			// ButtonApply
			// 
			this.ButtonApply.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.ButtonApply.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.ButtonApply.Enabled = false;
			this.ButtonApply.Location = new System.Drawing.Point(392, 244);
			this.ButtonApply.Name = "ButtonApply";
			this.ButtonApply.TabIndex = 1;
			this.ButtonApply.Text = "Apply";
			this.ButtonApply.Click += new System.EventHandler(this.ButtonApply_Click);
			// 
			// ButtonOK
			// 
			this.ButtonOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.ButtonOK.Location = new System.Drawing.Point(216, 244);
			this.ButtonOK.Name = "ButtonOK";
			this.ButtonOK.TabIndex = 2;
			this.ButtonOK.Text = "OK";
			this.ButtonOK.Click += new System.EventHandler(this.ButtonOK_Click);
			// 
			// ButtonSave
			// 
			this.ButtonSave.Location = new System.Drawing.Point(96, 16);
			this.ButtonSave.Name = "ButtonSave";
			this.ButtonSave.TabIndex = 3;
			this.ButtonSave.Text = "Save...";
			this.ButtonSave.Click += new System.EventHandler(this.ButtonSave_Click);
			// 
			// ButtonLoad
			// 
			this.ButtonLoad.Location = new System.Drawing.Point(16, 16);
			this.ButtonLoad.Name = "ButtonLoad";
			this.ButtonLoad.TabIndex = 4;
			this.ButtonLoad.Text = "Load...";
			this.ButtonLoad.Click += new System.EventHandler(this.ButtonLoad_Click);
			// 
			// ButtonCancel
			// 
			this.ButtonCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.ButtonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.ButtonCancel.Location = new System.Drawing.Point(304, 244);
			this.ButtonCancel.Name = "ButtonCancel";
			this.ButtonCancel.TabIndex = 5;
			this.ButtonCancel.Text = "Cancel";
			this.ButtonCancel.Click += new System.EventHandler(this.ButtonCancel_Click);
			// 
			// FormSetup
			// 
			this.AcceptButton = this.ButtonOK;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.ButtonCancel;
			this.ClientSize = new System.Drawing.Size(480, 277);
			this.Controls.Add(this.ButtonCancel);
			this.Controls.Add(this.ButtonLoad);
			this.Controls.Add(this.ButtonSave);
			this.Controls.Add(this.ButtonOK);
			this.Controls.Add(this.ButtonApply);
			this.Controls.Add(this.ListSetup);
			this.Name = "FormSetup";
			this.Text = "Setup";
			this.ResumeLayout(false);

		}
		#endregion

		void RefreshList()
		{
			ListSetup.Items.Clear();

			// StartingNodes
			AddListItem("Starting Nodes", 
				Convert.ToString(NewSetup.StartingNodes), 
				"Number of nodes simulation starts with");

			// StartingWebCaches
			AddListItem("Starting WebCaches", 
				Convert.ToString(NewSetup.StartingWebCaches), 
				"Number of GWebCaches simulation starts with");

			ListSetup.Items.Add("");

			// NetworkTTL
			AddListItem("Network TTL", 
				Convert.ToString(NewSetup.NetworkTTL), 
				"Number of hops packet lives for on network");

			// NetworkInflux
			AddListItem("Network Influx", 
				Convert.ToString(NewSetup.NetworkInflux), 
				"1 in x chance of node joining network per second");

			// NetworkOutflux 
			AddListItem("Network Outflux", 
				Convert.ToString(NewSetup.NetworkOutflux), 
				"1 in x chance of node leaving network per second");

			// NetworkCapacity
			AddListItem("Network Capacity", 
				Convert.ToString(NewSetup.NetworkCapacity), 
				"Maximum number of nodes network can handle");
			
			// NetworkFirewalled
			AddListItem("Network Firewalled", 
				Convert.ToString(NewSetup.NetworkFirewalled), 
				"Percentange of network firewalled");

			ListSetup.Items.Add("");

			// NodeConnectMin
			AddListItem("Node Connect Min", 
				Convert.ToString(NewSetup.NodeConnectMin), 
				"Number of connections node tries to keep");
			
			// NodeConnectMax
			AddListItem("Node Connect Max", 
				Convert.ToString(NewSetup.NodeConnectMax), 
				"Maximum number of connections node will handle");
			
			// NodeAvoidTriangles
			AddListItem("Avoid Triagles", 
				Convert.ToString(NewSetup.NodeAvoidTriangles), 
				"Node activlely tries to prevent collisions within 2 hops");

			ListSetup.Items.Add("");

			// TrafficSim
			AddListItem("Traffic Sim", 
				Convert.ToString(NewSetup.TrafficSim), 
				"Simulate network traffic");

			// NodeBandwidthUp
			AddListItem("Bandwidth Min", 
				Convert.ToString(NewSetup.NodeBandwidthMin), 
				"Minimum bandwidth a node will be created with");

			// NodeBandwidthDown
			AddListItem("Bandwidth Max", 
				Convert.ToString(NewSetup.NodeBandwidthMax), 
				"Maximum bandwidth a node will be created with");

			// NodeQueryFreq
			AddListItem("Query Freq", 
				Convert.ToString(NewSetup.NodeQueryFreq), 
				"Chance per second node sends packet (40 bytes)");

			ListSetup.Items.Add("");

			// UltrapeerSim
			AddListItem("Ultrapeer Sim", 
				Convert.ToString(NewSetup.UltrapeerSim), 
				"Simulate ultrapeer system");

			// UltrapeerCapacity
			AddListItem("Leaf Min", 
				Convert.ToString(NewSetup.NodeLeafMin), 
				"Minimum leaf capacity a node will be created with");

			AddListItem("Leaf Max", 
				Convert.ToString(NewSetup.NodeLeafMax), 
				"Maximum leaf capacity a node will be created with");

			// LeafModeConnects
			AddListItem("Leaf Mode Connects", 
				Convert.ToString(NewSetup.LeafModeConnects), 
				"Number of ultrapeer connects each leaf keeps");
		}

		void AddListItem(string variable, string varValue, string description)
		{
			ListViewItem variableItem = new ListViewItem(variable);
			variableItem.SubItems.Add( varValue );
			variableItem.SubItems.Add( description);

			ListSetup.Items.Add( variableItem);
		}

		private void ListSetup_DoubleClick(object sender, System.EventArgs e)
		{
			string variableName;
			string variableValue;
			int variableIndex;


			if( ListSetup.SelectedItems.Count > 0)
			{
				ListViewItem selectedItem = ListSetup.SelectedItems[0];
			
				variableName = selectedItem.Text;
				variableIndex = selectedItem.Index;
		
				if(variableName == "")
					return;

				ListViewItem.ListViewSubItem selectedSubItem = selectedItem.SubItems[1];
				variableValue = selectedSubItem.Text;
			}	
			else 
				return;

			FormSetupEdit editDialog = new FormSetupEdit();

			editDialog.Text = variableName;
			editDialog.TextValue.Text = variableValue;
			
			if( editDialog.ShowDialog(this) == DialogResult.OK)
			{
				if( !SetValue(variableIndex, editDialog.TextValue.Text))
					MessageBox.Show(this, "Invalid Value");
				else
					ButtonApply.Enabled = true;
			}
		}

		bool SetValue(int index, string strValue)
		{
			try
			{
				switch((IndexVars) index)
				{
					case IndexVars.StartingNodes:
						NewSetup.StartingNodes = Convert.ToInt32(strValue);
						break;
				
					case IndexVars.StartingWebCaches:
						NewSetup.StartingWebCaches = Convert.ToInt32(strValue);
						break;

					case IndexVars.NetworkTTL:
						NewSetup.NetworkTTL = Convert.ToInt32(strValue);
						break;

					case IndexVars.NetworkInflux:
						NewSetup.NetworkInflux = Convert.ToInt32(strValue);
						break;

					case IndexVars.NetworkOutflux:
						NewSetup.NetworkOutflux = Convert.ToInt32(strValue);
						break;

					case IndexVars.NetworkCapacity:
						NewSetup.NetworkCapacity = Convert.ToInt32(strValue);
						break;

					case IndexVars.NetworkFirewalled:
						NewSetup.NetworkFirewalled = Convert.ToInt32(strValue);
						break;

					case IndexVars.NodeConnectMin:
						NewSetup.NodeConnectMin = Convert.ToInt32(strValue);
						break;

					case IndexVars.NodeConnectMax:
						NewSetup.NodeConnectMax = Convert.ToInt32(strValue);
						break;

					case IndexVars.NodeAvoidTriangles:
						NewSetup.NodeAvoidTriangles = Convert.ToBoolean(strValue);
						break;

					case IndexVars.TrafficSim:
						NewSetup.TrafficSim = Convert.ToBoolean(strValue);
						break;

					case IndexVars.NodeBandwidthMin:
						NewSetup.NodeBandwidthMin = Convert.ToInt32(strValue);
						break;

					case IndexVars.NodeBandwidthMax:
						NewSetup.NodeBandwidthMax = Convert.ToInt32(strValue);
						break;

					case IndexVars.NodeQueryFreq:
						NewSetup.NodeQueryFreq = Convert.ToInt32(strValue);
						break;

					case IndexVars.UltrapeerSim:
						NewSetup.UltrapeerSim = Convert.ToBoolean(strValue);
						break;

					case IndexVars.NodeLeafMin:
						NewSetup.NodeLeafMin = Convert.ToInt32(strValue);
						break;

					case IndexVars.NodeLeafMax:
						NewSetup.NodeLeafMax = Convert.ToInt32(strValue);
						break;

					case IndexVars.LeafModeConnects:
						NewSetup.LeafModeConnects = Convert.ToInt32(strValue);
						break;
				}
			}
			catch
			{
				return false;
			}

			RefreshList();

			return true;
		}

		private void ButtonOK_Click(object sender, System.EventArgs e)
		{
			MainView.TestSetup = (SimSetup) NewSetup.Clone();
			MainView.TestNet.Settings = MainView.TestSetup;

			Close();
		}

		private void ButtonCancel_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void ButtonApply_Click(object sender, System.EventArgs e)
		{
			MainView.TestSetup = (SimSetup) NewSetup.Clone();
			MainView.TestNet.Settings = MainView.TestSetup;

			ButtonApply.Enabled = false;
		}

		private void ButtonSave_Click(object sender, System.EventArgs e)
		{
			Stream saveStream ;
			SaveFileDialog saveDialog = new SaveFileDialog();
 
			saveDialog.Filter = "gns files (*.gns)|*.gns|All files (*.*)|*.*"  ;
			saveDialog.FilterIndex = 1 ;
			saveDialog.RestoreDirectory = true ;
 
			if(saveDialog.ShowDialog() == DialogResult.OK)
			{
				if((saveStream = saveDialog.OpenFile()) != null)
				{
					XmlTextWriter xmlWriter = new XmlTextWriter(saveStream, Encoding.UTF8);

					xmlWriter.Formatting = Formatting.Indented;

					xmlWriter.WriteStartDocument();

					xmlWriter.WriteStartElement("GnucNS");
					xmlWriter.WriteStartElement("Setup");

					xmlWriter.WriteElementString("StartingNodes", XmlConvert.ToString(NewSetup.StartingNodes));
					xmlWriter.WriteElementString("StartingWebCaches", XmlConvert.ToString(NewSetup.StartingWebCaches));
	
					xmlWriter.WriteElementString("NetworkTTL", XmlConvert.ToString(NewSetup.NetworkTTL));
					xmlWriter.WriteElementString("NetworkInflux", XmlConvert.ToString(NewSetup.NetworkInflux));
					xmlWriter.WriteElementString("NetworkOutflux", XmlConvert.ToString(NewSetup.NetworkOutflux));
					xmlWriter.WriteElementString("NetworkCapacity", XmlConvert.ToString(NewSetup.NetworkCapacity));
					xmlWriter.WriteElementString("NetworkFirewalled", XmlConvert.ToString(NewSetup.NetworkFirewalled));
					
					xmlWriter.WriteElementString("NodeConnectMin", XmlConvert.ToString(NewSetup.NodeConnectMin));
					xmlWriter.WriteElementString("NodeConnectMax", XmlConvert.ToString(NewSetup.NodeConnectMax));
					xmlWriter.WriteElementString("NodeAvoidTriangles", XmlConvert.ToString(NewSetup.NodeAvoidTriangles));

					xmlWriter.WriteElementString("TrafficSim", XmlConvert.ToString(NewSetup.TrafficSim));
					xmlWriter.WriteElementString("NodeBandwidthMin", XmlConvert.ToString(NewSetup.NodeBandwidthMin));
					xmlWriter.WriteElementString("NodeBandwidthMax", XmlConvert.ToString(NewSetup.NodeBandwidthMax));
					xmlWriter.WriteElementString("NodeQueryFreq", XmlConvert.ToString(NewSetup.NodeQueryFreq));

					xmlWriter.WriteElementString("UltrapeerSim", XmlConvert.ToString(NewSetup.UltrapeerSim));
					xmlWriter.WriteElementString("NodeLeafMin", XmlConvert.ToString(NewSetup.NodeLeafMin));
					xmlWriter.WriteElementString("NodeLeafMax", XmlConvert.ToString(NewSetup.NodeLeafMax));
					xmlWriter.WriteElementString("LeafModeConnects", XmlConvert.ToString(NewSetup.LeafModeConnects));


					xmlWriter.WriteEndElement();
					xmlWriter.WriteEndElement();

					xmlWriter.WriteEndDocument();

					xmlWriter.Flush();
					saveStream.Close();
				}
			}

		}

		private void ButtonLoad_Click(object sender, System.EventArgs e)
		{
			Stream loadStream;
			OpenFileDialog loadDialog = new OpenFileDialog();

			loadDialog.Filter = "gns files (*.gns)|*.gns|All files (*.*)|*.*" ;
			loadDialog.FilterIndex = 1 ;
			loadDialog.RestoreDirectory = true ;

			if(loadDialog.ShowDialog() == DialogResult.OK)
			{
				if((loadStream = loadDialog.OpenFile())!= null)
				{
					XmlTextReader xmlReader = new XmlTextReader(loadStream);

					while( xmlReader.Read() )
					{
						if( xmlReader.IsStartElement("StartingNodes") )
							NewSetup.StartingNodes = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("StartingWebCaches") )
							NewSetup.StartingWebCaches = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NetworkTTL") )
							NewSetup.NetworkTTL = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NetworkInflux") )
							NewSetup.NetworkInflux = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NetworkOutflux") )
							NewSetup.NetworkOutflux = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NetworkCapacity") )
							NewSetup.NetworkCapacity = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NetworkFirewalled") )
							NewSetup.NetworkFirewalled = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeConnectMin") )
							NewSetup.NodeConnectMin = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeConnectMax") )
							NewSetup.NodeConnectMax = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeAvoidTriangles") )
							NewSetup.NodeAvoidTriangles = XmlConvert.ToBoolean( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("TrafficSim") )
							NewSetup.TrafficSim = XmlConvert.ToBoolean( xmlReader.ReadElementString() );
					
						if( xmlReader.IsStartElement("NodeBandwidthMin") )
							NewSetup.NodeBandwidthMin = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeBandwidthMax") )
							NewSetup.NodeBandwidthMax = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeQueryFreq") )
							NewSetup.NodeQueryFreq = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("UltrapeerSim") )
							NewSetup.UltrapeerSim = XmlConvert.ToBoolean( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeLeafMin") )
							NewSetup.NodeLeafMin = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("NodeLeafMax") )
							NewSetup.NodeLeafMax = XmlConvert.ToInt32( xmlReader.ReadElementString() );

						if( xmlReader.IsStartElement("LeafModeConnects") )
							NewSetup.LeafModeConnects = XmlConvert.ToInt32( xmlReader.ReadElementString() );

					}

					loadStream.Close();
					ButtonApply.Enabled = true;
				}
			}

			RefreshList();
		}
	}
}
