/********************************************************************************

	GnucNS - Network Simulator
    Copyright (C) 2003 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: swabby@c0re.net

********************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using GnucNS.Simulation;

namespace GnucNS.Interface
{
	/// <summary>
	/// Summary description for FormStats.
	/// </summary>
	public class FormStatsHorizon : System.Windows.Forms.Form
	{
		FormMain MainView;


		private System.Windows.Forms.Button ButtonCalculate;
		private System.Windows.Forms.Label LabelReach;
		private System.Windows.Forms.Label LabelCollisions;
		private System.Windows.Forms.ProgressBar ProgressCalc;
		private System.Windows.Forms.Label label1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public FormStatsHorizon(FormMain parent)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			MainView = parent;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			MainView.StatsHorizonView = null;

			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.LabelReach = new System.Windows.Forms.Label();
			this.LabelCollisions = new System.Windows.Forms.Label();
			this.ButtonCalculate = new System.Windows.Forms.Button();
			this.ProgressCalc = new System.Windows.Forms.ProgressBar();
			this.label1 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// LabelReach
			// 
			this.LabelReach.Location = new System.Drawing.Point(24, 48);
			this.LabelReach.Name = "LabelReach";
			this.LabelReach.Size = new System.Drawing.Size(288, 23);
			this.LabelReach.TabIndex = 0;
			this.LabelReach.Text = "Reach: ";
			// 
			// LabelCollisions
			// 
			this.LabelCollisions.Location = new System.Drawing.Point(24, 72);
			this.LabelCollisions.Name = "LabelCollisions";
			this.LabelCollisions.Size = new System.Drawing.Size(288, 24);
			this.LabelCollisions.TabIndex = 2;
			this.LabelCollisions.Text = "Collisions:";
			// 
			// ButtonCalculate
			// 
			this.ButtonCalculate.Location = new System.Drawing.Point(120, 104);
			this.ButtonCalculate.Name = "ButtonCalculate";
			this.ButtonCalculate.Size = new System.Drawing.Size(75, 24);
			this.ButtonCalculate.TabIndex = 3;
			this.ButtonCalculate.Text = "Calculate";
			this.ButtonCalculate.Click += new System.EventHandler(this.ButtonCalculate_Click);
			// 
			// ProgressCalc
			// 
			this.ProgressCalc.Location = new System.Drawing.Point(0, 128);
			this.ProgressCalc.Name = "ProgressCalc";
			this.ProgressCalc.Size = new System.Drawing.Size(328, 16);
			this.ProgressCalc.TabIndex = 7;
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 24);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(100, 16);
			this.label1.TabIndex = 9;
			this.label1.Text = "Average";
			// 
			// FormStatsHorizon
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(322, 143);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.ProgressCalc);
			this.Controls.Add(this.ButtonCalculate);
			this.Controls.Add(this.LabelCollisions);
			this.Controls.Add(this.LabelReach);
			this.Cursor = System.Windows.Forms.Cursors.Default;
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "FormStatsHorizon";
			this.Text = "Stats Horizon";
			this.ResumeLayout(false);

		}
		#endregion

		private void ButtonCalculate_Click(object sender, System.EventArgs e)
		{
			SimNetwork testNet = MainView.TestNet;
			SimSetup testSetup = MainView.TestSetup;

			int scanned = 0;

			int ultrapeers = 0;

			int  avgReach = 0;
			int  avgCollisions = 0;

			Cursor = Cursors.WaitCursor;
			ProgressCalc.Value = 0;

			IDictionaryEnumerator enumNode = testNet.NodeTable.GetEnumerator();
			while( enumNode.MoveNext() )
			{
				scanned++;
				ProgressCalc.Value = scanned * 100 / testNet.NodeTable.Count;

				SimNode currentNode = (SimNode) enumNode.Value;

				if( currentNode.ChildMode )
					continue;
				else
					ultrapeers++;

				Hashtable coveredNodes = new Hashtable(); // key nodeID, value hops
				ArrayList pendingNodes = new ArrayList();

				int hops = 0;
				pendingNodes.Add( new NodeLink(currentNode.NodeID, 0) );

				while(hops <= testSetup.NetworkTTL && pendingNodes.Count > 0)
				{
					ArrayList nextNodes = new ArrayList();

					foreach( NodeLink link in pendingNodes)
					{
						SimNode node = (SimNode) testNet.NodeTable[link.NodeID];

						if( coveredNodes[link.NodeID] == null )
						{
							coveredNodes[link.NodeID] = hops;

							if(hops < testSetup.NetworkTTL)
								foreach( int nextNodeID in node.Connections)
								{
									SimNode nextNode = (SimNode) testNet.NodeTable[nextNodeID];

									if( nextNode != null && nextNodeID != link.LinkID)
										nextNodes.Add( new NodeLink(nextNodeID, link.NodeID) );
								}
						}
						else if(link.NodeID != 0)
						{
							avgCollisions++;
						}	
					}

					pendingNodes = nextNodes;
					hops++;
				}

				avgReach += coveredNodes.Count;
			}

			Cursor = Cursors.Default;

			if( testNet.NodeTable.Count > 0 )
			{
				avgReach     = (avgReach / ultrapeers) * 100 / ultrapeers;
				avgCollisions /= ultrapeers;
			}
			
			LabelReach.Text = string.Format("Reach: {0}%", avgReach);
			LabelCollisions.Text = string.Format("Collisions: {0}", avgCollisions);
		}



	}

}
