/********************************************************************************

	GnucNS - Network Simulator
    Copyright (C) 2003 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: swabby@c0re.net

********************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using GnucNS.Simulation;

namespace GnucNS.Interface
{
	/// <summary>
	/// Summary description for FormWebCache.
	/// </summary>
	public class FormWebCache : System.Windows.Forms.Form
	{
		FormMain MainView;

		private System.Windows.Forms.Panel PanelWebCache;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public FormWebCache(FormMain Parent)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			MainView = Parent;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			MainView.WebCacheView = null;

			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.PanelWebCache = new System.Windows.Forms.Panel();
			this.SuspendLayout();
			// 
			// PanelWebCache
			// 
			this.PanelWebCache.BackColor = System.Drawing.Color.White;
			this.PanelWebCache.Dock = System.Windows.Forms.DockStyle.Fill;
			this.PanelWebCache.Location = new System.Drawing.Point(0, 0);
			this.PanelWebCache.Name = "PanelWebCache";
			this.PanelWebCache.Size = new System.Drawing.Size(292, 273);
			this.PanelWebCache.TabIndex = 2;
			this.PanelWebCache.Resize += new System.EventHandler(this.PanelWebCache_Resize);
			this.PanelWebCache.Paint += new System.Windows.Forms.PaintEventHandler(this.PanelWebCache_Paint);
			// 
			// FormWebCache
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Controls.Add(this.PanelWebCache);
			this.Name = "FormWebCache";
			this.Text = "WebCache";
			this.ResumeLayout(false);

		}
		#endregion

		private void PanelWebCache_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			SimNetwork testNet = MainView.TestNet;

			Graphics display = e.Graphics;

			display.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;


			int	  nodeDiam		 = 6;
			Size  nodeSize		 = new Size(nodeDiam, nodeDiam);
			Point nodeCenter	 = new Point();
			SolidBrush nodeBrush = new SolidBrush(Color.Black);

			IDictionaryEnumerator nodeEnum = testNet.WebCacheTable.GetEnumerator();

			int rows = Convert.ToInt32( Math.Sqrt(testNet.WebCacheTable.Count) ) + 1;
			int cols = rows;

			for(int x = 1; x < rows; x++)
				for(int y = 1; y < cols; y++)
				{
					if( nodeEnum.MoveNext() )
					{
						nodeCenter.X = (PanelWebCache.Width  / rows) * y - (nodeDiam / 2);
						nodeCenter.Y = (PanelWebCache.Height / cols) * x - (nodeDiam / 2);
		
						display.FillEllipse(nodeBrush, new Rectangle(nodeCenter, nodeSize));

						SimWebCache currentNode = (SimWebCache) nodeEnum.Value;
						currentNode.DisplayPos = nodeCenter;
					}
					else
						break;
				}
		}

		

		private void PanelWebCache_Resize(object sender, System.EventArgs e)
		{
			PanelWebCache.Invalidate();
		}
	}
}
