/********************************************************************************

	GnucNS - Network Simulator
    Copyright (C) 2003 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: swabby@c0re.net

********************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using GnucNS.Simulation;

namespace GnucNS.Interface
{
	/// <summary>
	/// Summary description for NodeInfo.
	/// </summary>
	public class NodeInfo : System.Windows.Forms.Form
	{
		FormNetwork NetworkForm;
		private System.Windows.Forms.Label LabelFirewall;
		private System.Windows.Forms.Label LabelUptime;
		private System.Windows.Forms.Label LabelConnections;
		private System.Windows.Forms.Label LabelReach;
		private System.Windows.Forms.Label LabelCollisions;
		private System.Windows.Forms.Label LabelChildren;
		private System.Windows.Forms.Label LabelPacketQueue;
		private System.Windows.Forms.Label LabelBytesUp;
		private System.Windows.Forms.Label LabelBytesCollided;
		private System.Windows.Forms.Label LabelBytesDown;
		private System.Windows.Forms.Label LabelBytesDropped;
		private System.Windows.Forms.Label LabelBandwidth;
		private System.Windows.Forms.Label LabelCapacity;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public NodeInfo(FormNetwork parent)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			NetworkForm = parent;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			NetworkForm.DialogInfo = null;

			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		public void SetData(int nodeID, int coverage, int collisions)
		{
			SimNetwork testNet = NetworkForm.MainView.TestNet;
			SimNode node = (SimNode) testNet.NodeTable[nodeID];
			SimSetup settings = NetworkForm.MainView.TestSetup;

			if(node == null)
			{
				Text = string.Format("Node ID:{0} - Down", nodeID);

				LabelFirewall.Text = "";
				LabelUptime.Text   = "";
				LabelConnections.Text = "";
				LabelChildren.Text = "";
				LabelCapacity.Text = "";

				LabelReach.Text = "";
				LabelCollisions.Text = "";
				
				LabelBandwidth.Text = "";
				LabelBytesUp.Text = "";
				LabelBytesDown.Text = "";
				LabelBytesDropped.Text = "";
				LabelBytesCollided.Text = "";
				LabelPacketQueue.Text = "";
				

				return;
			}

			Text = string.Format("Node ID:{0}", node.NodeID);

			LabelFirewall.Text = string.Format("Firewall: {0}", node.Firewalled);
			LabelUptime.Text   = string.Format("Uptime: {0} Hours, {1} Mins, {2} Secs", node.NodeAlive / 3600, (node.NodeAlive % 3600) / 60, node.NodeAlive % 60);
			LabelConnections.Text = string.Format("Connections: {0}", node.Connections.Count);
			LabelChildren.Text = string.Format("Leaves: {0}", node.Children.Count);
			LabelCapacity.Text = string.Format("Leaf Capacity: {0}", node.LeafCapacity);
		
			LabelReach.Text = string.Format("Network Reach: {0}%", coverage);
			LabelCollisions.Text = string.Format("Collisions: {0}", collisions);

			
			LabelBandwidth.Text = string.Format("Bandwidth: {0}", node.BandwidthUp + node.BandwidthDown);
			if(node.LogBytesUp.Count > 0)
				LabelBytesUp.Text = string.Format("Bytes Up: {0}", node.TotBytesUp / node.LogBytesUp.Count);
			if(node.LogBytesDown.Count > 0)
				LabelBytesDown.Text = string.Format("Bytes Down: {0}", node.TotBytesDown / node.LogBytesDown.Count);
			if(node.LogBytesDropped.Count > 0)
				LabelBytesDropped.Text = string.Format("Bytes Dropped: {0}", node.TotBytesDropped / node.LogBytesDropped.Count);
			if(node.LogBytesCollided.Count > 0)
				LabelBytesCollided.Text = string.Format("Bytes Collided (wasted): {0}", node.TotBytesCollided / node.LogBytesCollided.Count);
			
			LabelPacketQueue.Text = string.Format("Packet Queue: {0}", node.PacketQueue.Count);
		
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.LabelFirewall = new System.Windows.Forms.Label();
			this.LabelUptime = new System.Windows.Forms.Label();
			this.LabelConnections = new System.Windows.Forms.Label();
			this.LabelReach = new System.Windows.Forms.Label();
			this.LabelCollisions = new System.Windows.Forms.Label();
			this.LabelChildren = new System.Windows.Forms.Label();
			this.LabelPacketQueue = new System.Windows.Forms.Label();
			this.LabelBytesUp = new System.Windows.Forms.Label();
			this.LabelBytesCollided = new System.Windows.Forms.Label();
			this.LabelBytesDown = new System.Windows.Forms.Label();
			this.LabelBytesDropped = new System.Windows.Forms.Label();
			this.LabelBandwidth = new System.Windows.Forms.Label();
			this.LabelCapacity = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// LabelFirewall
			// 
			this.LabelFirewall.Location = new System.Drawing.Point(16, 32);
			this.LabelFirewall.Name = "LabelFirewall";
			this.LabelFirewall.Size = new System.Drawing.Size(192, 16);
			this.LabelFirewall.TabIndex = 0;
			this.LabelFirewall.Text = "Firewall:";
			// 
			// LabelUptime
			// 
			this.LabelUptime.Location = new System.Drawing.Point(16, 16);
			this.LabelUptime.Name = "LabelUptime";
			this.LabelUptime.Size = new System.Drawing.Size(192, 16);
			this.LabelUptime.TabIndex = 1;
			this.LabelUptime.Text = "Uptime";
			// 
			// LabelConnections
			// 
			this.LabelConnections.Location = new System.Drawing.Point(16, 48);
			this.LabelConnections.Name = "LabelConnections";
			this.LabelConnections.Size = new System.Drawing.Size(192, 16);
			this.LabelConnections.TabIndex = 2;
			this.LabelConnections.Text = "Connections";
			// 
			// LabelReach
			// 
			this.LabelReach.Location = new System.Drawing.Point(16, 104);
			this.LabelReach.Name = "LabelReach";
			this.LabelReach.Size = new System.Drawing.Size(192, 16);
			this.LabelReach.TabIndex = 3;
			this.LabelReach.Text = "Network Reach";
			// 
			// LabelCollisions
			// 
			this.LabelCollisions.Location = new System.Drawing.Point(16, 120);
			this.LabelCollisions.Name = "LabelCollisions";
			this.LabelCollisions.Size = new System.Drawing.Size(192, 16);
			this.LabelCollisions.TabIndex = 4;
			this.LabelCollisions.Text = "Collisions:";
			// 
			// LabelChildren
			// 
			this.LabelChildren.Location = new System.Drawing.Point(16, 64);
			this.LabelChildren.Name = "LabelChildren";
			this.LabelChildren.Size = new System.Drawing.Size(192, 16);
			this.LabelChildren.TabIndex = 5;
			this.LabelChildren.Text = "Leaves:";
			// 
			// LabelPacketQueue
			// 
			this.LabelPacketQueue.Location = new System.Drawing.Point(16, 224);
			this.LabelPacketQueue.Name = "LabelPacketQueue";
			this.LabelPacketQueue.Size = new System.Drawing.Size(192, 15);
			this.LabelPacketQueue.TabIndex = 14;
			this.LabelPacketQueue.Text = "Packet Queue:";
			// 
			// LabelBytesUp
			// 
			this.LabelBytesUp.Location = new System.Drawing.Point(16, 160);
			this.LabelBytesUp.Name = "LabelBytesUp";
			this.LabelBytesUp.Size = new System.Drawing.Size(192, 15);
			this.LabelBytesUp.TabIndex = 13;
			this.LabelBytesUp.Text = "Bytes Up:";
			// 
			// LabelBytesCollided
			// 
			this.LabelBytesCollided.Location = new System.Drawing.Point(16, 208);
			this.LabelBytesCollided.Name = "LabelBytesCollided";
			this.LabelBytesCollided.Size = new System.Drawing.Size(192, 15);
			this.LabelBytesCollided.TabIndex = 12;
			this.LabelBytesCollided.Text = "Bytes Collided (wasted):";
			// 
			// LabelBytesDown
			// 
			this.LabelBytesDown.Location = new System.Drawing.Point(16, 176);
			this.LabelBytesDown.Name = "LabelBytesDown";
			this.LabelBytesDown.Size = new System.Drawing.Size(192, 15);
			this.LabelBytesDown.TabIndex = 11;
			this.LabelBytesDown.Text = "Bytes Down:";
			// 
			// LabelBytesDropped
			// 
			this.LabelBytesDropped.Location = new System.Drawing.Point(16, 192);
			this.LabelBytesDropped.Name = "LabelBytesDropped";
			this.LabelBytesDropped.Size = new System.Drawing.Size(192, 15);
			this.LabelBytesDropped.TabIndex = 10;
			this.LabelBytesDropped.Text = "Bytes Dropped:";
			// 
			// LabelBandwidth
			// 
			this.LabelBandwidth.Location = new System.Drawing.Point(16, 144);
			this.LabelBandwidth.Name = "LabelBandwidth";
			this.LabelBandwidth.Size = new System.Drawing.Size(192, 15);
			this.LabelBandwidth.TabIndex = 15;
			this.LabelBandwidth.Text = "Bandwidth:";
			// 
			// LabelCapacity
			// 
			this.LabelCapacity.Location = new System.Drawing.Point(16, 80);
			this.LabelCapacity.Name = "LabelCapacity";
			this.LabelCapacity.Size = new System.Drawing.Size(192, 16);
			this.LabelCapacity.TabIndex = 16;
			this.LabelCapacity.Text = "Leaf Capacity:";
			// 
			// NodeInfo
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(218, 250);
			this.Controls.Add(this.LabelCapacity);
			this.Controls.Add(this.LabelBandwidth);
			this.Controls.Add(this.LabelPacketQueue);
			this.Controls.Add(this.LabelBytesUp);
			this.Controls.Add(this.LabelBytesCollided);
			this.Controls.Add(this.LabelBytesDown);
			this.Controls.Add(this.LabelBytesDropped);
			this.Controls.Add(this.LabelChildren);
			this.Controls.Add(this.LabelCollisions);
			this.Controls.Add(this.LabelReach);
			this.Controls.Add(this.LabelConnections);
			this.Controls.Add(this.LabelUptime);
			this.Controls.Add(this.LabelFirewall);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			this.Name = "NodeInfo";
			this.ShowInTaskbar = false;
			this.Text = "Node ID";
			this.Load += new System.EventHandler(this.NodeInfo_Load);
			this.ResumeLayout(false);

		}
		#endregion

		private void NodeInfo_Load(object sender, System.EventArgs e)
		{
		
		}
	}
}
